;
; md202006 - get well soon
;
; released by cosine in 2020
;
; cosine.org.uk
;
; code & graphics by
; aNdy (CSDb id=6404)
;
; music by
; odie (CSDb id=1181)
;
; this source is formatted for the ACME cross assembler
; which can be downloaded from...
;
; http://sourceforge.net/projects/acme-crossass/
;
; the final build was compressed and squished using
; exomizer 2 which can be downloaded from...
;
; http://hem.bredband.net/magli143/exo/

; please note, there is reference to 'twister'
; throughout the code because the 'wibbly' thing
; was originally going to be a twister, but changed
; and i was too lazy to update all the references... :D
;

; the name and file format for the assembled program

		!to "sourcebuild.prg",cbm			; gimme commodore!
		
		!cpu 6510							; some illegal opcodes liveth here!

; claw in the binary data...
		
		* = $0900
music	!binary "data/firepower.bin",,2		; the tune, called 'firepower'

		* = $2000
logo	!binary "data/logo.chr"				; cosine logo charset
		
		* = $2800
		!binary "data/sprites.spr"			; copyright symbol / get well sprites
		
		* = $3000
		!binary "data/font.chr"				; scrolling message charset 'font'
		
		* = $3800
		!binary "data/twister.chr"			; charset for the fake 'twister'
		
; position of the raster interrupt(s) on-screen

raster_1_pos	= $00			; top border
raster_2_pos	= $32			; start of logo
raster_3_pos	= $7a			; bottom of logo
raster_4_pos	= $ca			; bottom of twister
raster_5_pos	= $e1			; end of scroller
raster_6_pos	= $f9			; bottom border

; some more label assignments / variables

raster_num			= $50				; raster handler number
sync				= $51				; interupt sync variable

sid_type			= $52				; store for SID type detection code

msb_flag1			= $53				; flag for setting logo sprites msb
msb_flag2			= $54				; flag for setting twister sprites msb

scroll_x			= $55				; store for scroller x position
scroll_speed		= $56				; store for scroller speed
scroll_width		= $57				; store for character width

typer_x				= $58				; store for typer x
typer_pos			= $59				; store for typer position - $02 bytes used!

twister_tmr			= $5b				; store twister timer
twister_sel			= $5c				; store for twister pattern selection

greets_txtcol		= $5d				; store for greets text colour
greets_coltmr		= $5e				; timer for greets colour flash

; add a BASIC startline (SYS 16384) for auto-run after loading

		* = $0801
		
		!word code_entry-2
		!byte $00,$00,$9e
		!text "16384"
		!byte $00,$00,$00

; entry point for the code

		* = $4000					; hence sys 16384!
				
code_entry

; stop interrupts, disable the roms and set up nmi and irq interrupt pointers	

		sei							; poke the interrupts in the eye temporarily
		
		ldx #$40
		lda #$00
clr_zp	
		sta $00,x					; clear out the zero page in case...
		inx
		bne clr_zp			

		lda #<nmi					; some nmi stuff
		sta $fffa
		lda #>nmi
		sta $fffb

		lda #<int					; some irq stuff
		sta $fffe
		lda #>int
		sta $ffff

		lda #$7f
		sta $dc0d					; disable timer interrupts generated by the two CIA chips
		sta $dd0d					; kernal uses these to flash the cursor and scan
									; the keyboard, so we better stop it

		lda $dc0d					; read these two registers to stop any CIA irqs
		lda $dd0d					; if we don't do this, a pending CIA irq might occur after we
									; finish setting up our irq - we don't want that to happen!

		lda #raster_1_pos			; starting point of raster on screen ($00)
		sta $d012

		lda #$1b					; as there are more than 256 rasterlines, the topmost bit of $d011 serves as
		sta $d011					; the 9th bit for the rasterline we want our irq to be triggered
		
		lda #$01
		sta $d019					; clear interupt bit
		sta $d01a					; tell vicii to generate interupt

		lda #$35					; bin the roms and kernal, we're flying solo cap'n
		sta $01

		lda #$03					; select video bank
		sta $dd00		
						
; detect which SID is being used and modify the music driver
; filter table if a 6581 is being used - provided by odie/cosine

		lda	#$20					; set frequency of SID channel 3 to #$2020
		sta	$d40e					; low frequency register
		sta	$d40f					; sid_type frequency register
		lda	#$31					; set channel 3 waveform to #$31 (mix of sawtooth and triangle waveform and gate on bit)
		sta	$d412					; channel 3 waveform register
		
		ldx	#$00					; set x to 0
		stx	sid_type				; clear value in variable 'sid_type'
sid_loop	
		lda	$d41b					; read oscillator 3 value in SID
		cmp	sid_type				; is it larger than value in 'sid_type'?
		bcc	sid_skip				; no, step over next command
		sta	sid_type				; yes, store new 'sid_type' value
sid_skip	
		dex							; decrease x
		bne	sid_loop				; loop 256 times
		lda	#$30					; triangle and sawtooth waveform gated off
		sta	$d412					; set waveform in channel 3

		lda	sid_type				; read "sid_type" variable
		bmi	sid_8580				; if there is a value #$80 or greater we have an 8580 chip
									; so no modification is needed - skip down to 'sid_8580'

; this part modifies the filter table for the music located at $0900
; when a 6581 sid chip is being used (thanks odie!)

		lda	#$38					; filter sweep from #$30 to #$80
		sta	$161f					; modify the start/end sweep range
		lda	#$08					; filter sweep speed in steps of #$08
		sta	$1620					; modify the sweep speed

sid_8580

		cli							; kickstart the interrupts again
		
; set up the music driver	

		lda #$00					; tune 0 loaded into the
		jsr music+$00				; music driver please!
		
; set up some labels		
		
		lda #$03					; starting speed for our scroller
		sta scroll_speed
		
		lda #$00
		sta twister_tmr				; twister timer to zero
		sta greets_txtcol			; greets colout text to zero
		sta greets_coltmr			; also zero the colour timer		
		
		lda #$01					; set twister pattern 2 to ready
		sta twister_sel				; after first is shown on start
		
; setup the screen for the very first run

		jsr clear_screen			; clear screen with subroutine

; draw the cosine logo at the top of the screen	
		
		ldx #$00
logodraw
		lda logo_matrix,x			; draw logo using data from the
		sta $0400,x					; logo matrix in the data
		lda logo_matrix+40,x		; section at the bottom of this code
		sta $0400+40,x
		lda logo_matrix+80,x
		sta $0400+80,x	
		lda logo_matrix+120,x
		sta $0400+120,x							
		lda logo_matrix+160,x
		sta $0400+160,x			
		lda logo_matrix+200,x
		sta $0400+200,x			
		lda logo_matrix+240,x
		sta $0400+240,x
		lda logo_matrix+280,x
		sta $0400+280,x
		lda logo_matrix+320,x
		sta $0400+320,x								
		
		lda #$01					; colour the whole logo white
		sta $d800,x
		sta $d800+40,x
		sta $d800+80,x
		sta $d800+120,x
		sta $d800+160,x						
		sta $d800+200,x
		sta $d800+240,x
		sta $d800+280,x
		sta $d800+320,x						
		inx
		cpx #$28
		bne logodraw
		
; draw the twister thingie in the middle of the screen

		ldx #$00
twisterdraw		
		lda twister_matrix,x		; draw twister using data from the matrix
		sta $05b8,x
		sta $05b8-40,x		
		sta $05b8+40,x
		sta $05b8+80,x	
		sta $05b8+120,x	
		sta $05b8+160,x	
		sta $05b8+200,x
		sta $05b8+240,x		
		
		lda #$00
		sta $05b8+320,x														

		lda #$03					; add some colour for each row
		sta $d9b8,x
		lda #$0d		
		sta $d9b8+40,x
		lda #$03																				
		sta $d9b8+80,x
		lda #$0e	
		sta $d9b8-40,x		
		sta $d9b8+120,x
		lda #$04				
		sta $d9b8+160,x
		lda #$0b				
		sta $d9b8+200,x
		lda #$06		
		sta $d9b8+240,x
		
		lda #$01				
		sta $d9b8+320,x
				
		inx
		cpx #$27
		bne twisterdraw	

		
; finally, set up the bottom third of screen for the scroller			
		
		ldx #$00
scrollerdraw		
		lda #$20					; rest of the screen is solid
		sta $06f8+40,x
		sta $06f8+80,x
		sta $06f8+120,x	
		sta $06f8+160,x		
		sta $06f8+200,x
		
		lda #$01					; and all this is white
		sta $daf8+40,x
		sta $daf8+80,x
		sta $daf8+120,x
		sta $daf8+160,x
		sta $daf8+200,x

		inx
		cpx #$28
		bne scrollerdraw
		
; reset the scrolling and typing messages to the begining

		jsr reset_scroller
		
		jsr reset_typer			
						
; main code lives here! =================================
		
mainloop

		jsr sync_wait			; wait for irq to run their course...

		jsr type_writer			; type the scene groups greetings text
				
		jsr twister_updater		; update the twister pattern and animation

		lda $dc01				; load spacebar register
		cmp #$ef				; is space pressed?
		beq *+$05				; yes? jump down to reset the 64...

		jmp mainloop			; no? go back to 'mainloop' up by there
								; and call the subroutines again!
								
; space has been pressed so reset some registers

		sei						; stop the interupts
		
		lda #$37				; restart roms / kernal
		sta $01

		lda #$00				; reset these registers
		sta $d011
		sta $d020
		sta $d021
		sta $d418

; then reset the C64

		jmp $fce2				; rom rest call		
												

; irq interrupt code =================================

int
		pha  
		txa
		pha
		tya
		pha
		
		lda $d019
		and #$01
		sta $d019
		bne intr_trigd
		
		jmp irq_exit			; no interupt triggered? exit irq code!
		
; an interrupt has triggered!?!
; which irq split should we run? let's check...

intr_trigd

		lda raster_num			; load up the raster_num variable
		
		cmp #$02				; is it set to '2'?
		bne *+$05				; no? then jump down to check for '3'
		jmp irq_rout2			; yes? jump down to irq_rout2!

		cmp #$03				; is the raster_num set to '3'?
		bne *+$05				; no? jump down and run irq_rout1!
		jmp irq_rout3			; yes? jump down to irq_rout3!

		cmp #$04				; is the raster_num set to '4'?
		bne *+$05				; no? jump down and run irq_rout1!
		jmp irq_rout4			; yes? jump down to irq_rout4!
		
		cmp #$05				; is the raster_num set to '5'?
		bne *+$05				; no? jump down and run irq_rout1!
		jmp irq_rout5			; yes? jump down to irq_rout5!
		
		cmp #$06				; is the raster_num set to '6'?
		bne *+$05				; no? jump down and run irq_rout1!
		jmp irq_rout6			; yes? jump down to irq_rout6!		

; raster split 1 code -------------------------------

irq_rout1

; set interrupt handler ready for split 2
		
		lda #$02
		sta raster_num
		lda #raster_2_pos
		sta $d012
		
; split 1 is done, exit irq interrupt		
					
		jmp irq_exit				; now exit the interupt!
		
			
; raster split 2 code ------------------------------------

irq_rout2

		lda #$18					; use our logo charset
		sta $d018					; at $2000
	
		lda #$00					; black background please!
		sta $d021
	
; set up the hardware sprites for the logo section
		
		lda #$ff					; turn on..
		sta $d015					; all 8 sprites!
		
		lda #$00					; make these sprites
		sta $d01c					; hires
		
		lda #$ff					; put the sprites behind
		sta $d01b					; the logo

		sta $d01d					; expand all sprites 
;		sta $d017					; horizontally & vertically

; print the sprites on the screen

		ldx #$00
		ldy #$00
logo_sprites						; lets draw the message sprites
		lda logo_sprites_x,x		; load in the sprite x position from the table
		sta $d000,y					; store in the sprites x register
		lda logo_sprites_msb		; check if we're in the MSB
		sta $d010			
		lda logo_sprites_y			; pull in the y position
		sta $d001,y
		lda #$0c					; make sprites grey
		sta $d027,x
		lda logo_sprites_def,x		; pull in sprite definitions from the defs table
		sta $07f8,x
		iny
		iny
		inx
		cpx #$05					; do the above 5 times because we're using
		bne logo_sprites			; 5 sprites, funnily enough...
			
		ldx #$00
		lda #$01
		sta msb_flag1				; set the temp value used for MSB juggling

; now move the sprites across the screen!		
				
reposition_logo_sprites
		lda logo_sprites_x,x		; pull in the x position of the sprites
		sbc logo_sprites_s,x		; and subtract a distance in pixels
		sta logo_sprites_x,x		; then store back to the x position table

		bcs no_msb_logo				; skip the MSB wrangling if carry isn't set

		lda logo_sprites_msb		; read the current MSB value
		eor msb_flag1				; toggle the bit for the sprite being processed
		sta logo_sprites_msb		; write the current MSB value

no_msb_logo
		asl msb_flag1				; shift the bit used for toggling MSB up for the next pass
		inx
		cpx #$05				
		bne reposition_logo_sprites
		
; now alter the logo sprites y position to 'bounce' them

logo_sprites_y_upd
		lda logo_sprites_ydir		; which y direction are the sprites moving?
		cmp #$01					; up?
		beq logo_sprites_up			; yes? branch to logo sprites up!
									; no?	
		lda logo_sprites_y			; then move the sprites down!
		clc
		adc #$01
		sta logo_sprites_y
		cmp #$50
		bne logo_spritey_dun
		lda #$01
		sta logo_sprites_ydir
		jmp logo_spritey_dun
					
logo_sprites_up
		lda logo_sprites_y
		sec
		sbc #$01
		sta logo_sprites_y		
		cmp #$3d
		bne logo_spritey_dun
		lda #$00
		sta logo_sprites_ydir

logo_spritey_dun

; call music player	and play dat choon!
		
		jsr music+$03			; play odie's tune!

; set interrupt handler ready for split 3
	
		lda #$03
		sta raster_num
		lda #raster_3_pos
		sta $d012
	
; split 2 is done, exit irq interrupt

		jmp irq_exit		

				
; raster split 3 code ------------------------------------

irq_rout3
		
		lda #$1e					; use our twister charset
		sta $d018					; at $3800

; set up the hardware sprites for the twister section
		
		lda #$ff					; turn on..
		sta $d015					; all 8 sprites!
		
		lda #$00					; make these sprites
		sta $d01c					; hires
		sta $d01b					; put the sprites in front
									; of the twister and
		sta $d01d					; unexpand all sprites 
		sta $d017					; horizontally & vertically

; print the copyright sprites on the screen

		ldx #$00
		ldy #$00
twister_sprites						; lets draw the cosine copyright symbol
		lda twister_sprites_x,x		; load in the sprite x position from the table
		sta $d000,y					; store in the sprites x register
		lda twister_sprites_msb		; check if we're in the MSB
		sta $d010			
		lda twister_sprites_y,x		; pull in the y position
		sta $d001,y
		lda twister_sprites_col,x	; pull in colours
		sta $d027,x
		lda twister_sprites_def,x	; pull in sprite definitions from the defs table
		sta $07f8,x
		iny
		iny
		inx
		cpx #$08
		bne twister_sprites
			
		ldx #$00
		lda #$01
		sta msb_flag2				; set the temp value used for MSB juggling		

; now move the copyright symbol sprites	over the twister	

		lda twister_sprites_xdir
		cmp #$00
		bne twister_sprites_right
				
twister_sprites_left

		lda twister_sprites_x,x		; pull in the x position of the sprites
		sbc twister_sprites_sl,x	; and subtract a distance in pixels
		sta twister_sprites_x,x		; then store back to the x position table

		bcs no_msb_twister1			; skip the MSB wrangling if carry isn't set

		lda twister_sprites_msb		; read the current MSB value
		eor msb_flag2				; toggle the bit for the sprite being processed
		sta twister_sprites_msb		; write the current MSB value

no_msb_twister1
		asl msb_flag2				; shift the bit used for toggling MSB up for the next pass
		inx
		cpx #$08				
		bne twister_sprites_left
		
		lda twister_sprites_x+$00
		cmp #$28
		bcs skipswitch
		
		lda #$01
		sta twister_sprites_xdir

skipswitch		
				
		jmp twister_sprites_dun
	
twister_sprites_right
		
		lda twister_sprites_x,x		; pull in the x position of the sprites
		adc twister_sprites_sr,x	; and add a distance in pixels
		sta twister_sprites_x,x		; then store back to the x position table

		bcc no_msb_twister2			; skip the MSB wrangling if carry isn't set

		lda twister_sprites_msb		; read the current MSB value
		eor msb_flag2				; toggle the bit for the sprite being processed
		sta twister_sprites_msb		; write the current MSB value

no_msb_twister2
		asl msb_flag2				; shift the bit used for toggling MSB up for the next pass
		inx
		cpx #$08				
		bne twister_sprites_right
		
		lda twister_sprites_x+$00
		cmp #$fe
		bcc twister_sprites_dun
		
		lda #$00
		sta twister_sprites_xdir		
				
twister_sprites_dun	

; set interrupt handler ready for split 4
	
		lda #$04
		sta raster_num
		lda #raster_4_pos
		sta $d012

; split 3 is done, exit irq interrupt

		jmp irq_exit			

		
; raster split 4 code ------------------------------------

irq_rout4

		lda #$1c					; use our font charset
		sta $d018					; at $3000

		lda scroll_x				; get video registers
		and #$07					; ready for the
		eor #$07					; scroller
		sta $d016

; setup sprites for the scroller screen edge 'faders'

		lda #%00000011				; turn on..
		sta $d015					; 2 sprites!
		
		lda #$00					; make these sprites
		sta $d01c					; hires
		sta $d01b					; put the sprites in front
									; of the scroller

		lda %00000001				; sprite 1
		sta $d010					; is in the MSB!									
																		
		ldx #$00
		ldy #$00
fader_sprites						; lets draw the sprites
		lda fader_sprites_x,x		; load in the x pos from the table
		sta $d000,y					; store in the sprites x register
		lda fader_sprites_y,x		; pull in y positions
		sta $d001,y					; store in the sprites y registers
		lda fader_sprites_col,x		; pull in sprite colours
		sta $d027,x					; and store in the colour registers
		lda fader_sprites_def,x		; pull in sprite definitions
		sta $07f8,x					; and throw them into memory
		iny
		iny
		inx
		cpx #$02					; both done?
		bne fader_sprites			; no? get back up there and do more!					

; set interrupt handler ready for split 5		

		lda #$05
		sta raster_num
		lda #raster_5_pos
		sta $d012	

; split 4 is done, exit irq interrupt

		jmp irq_exit

; raster split 5 code -------------------------------

irq_rout5

		lda #$00
		sta $d016

; choose background colour from colour table

		ldy greets_txtcol
		lda greets_colour,y		
		sta $d021
		
; time to change the above colour?		
		
		ldx greets_coltmr
		inx
		cpx #$03
		bne greets_tmr_skip			

		lda greets_txtcol
		inc greets_txtcol
		cmp #$07
		bne no_col_reset	
		
		lda #$00
		sta greets_txtcol

no_col_reset				
		ldx #$00
greets_tmr_skip
		stx greets_coltmr			
		
		
		

; update the scrolling message here even though the
; scroller is shown in the split above, otherwise it will
; glitch and tear while moving because the damn
; c64 can't draw the changes fast enough while 
; displying the thing!  in effect, we're updating
; the message after it is displayed so it's
; ready next pass through...

		ldy scroll_speed

scroll_loop
		ldx scroll_x
		inx
		cpx #$08
		bne sx_xb

		ldx #$00
mover	
		lda $0721,x
		sta $0720,x
		lda $0749,x
		sta $0748,x
		
		inx
		cpx #$26
		bne mover

		lda scroll_width
		beq mread

; no new character due
		
		inc	$0746
		inc $076e	
		
		dec scroll_width
		jmp sx_xb-$02

; new character due

mread	
		lda scroll_text
		bne okay
		jsr reset_scroller
		jmp mread

okay	
		cmp #$40
		bcc okay_2
		
		and #$0f
		sta scroll_speed		

		lda #$20
okay_2	
		tax
		lda char_pos_dcd,x
		sta $0746
		clc
		adc #$20
		sta $076e

		lda char_width_dcd,x
		sta scroll_width

		inc mread+$01
		bne *+$05
		inc mread+$02

		ldx #$00
sx_xb
		stx scroll_x
		
		dey
		bne scroll_loop
		
; set interrupt handler ready for split 6
		
		lda #$06
		sta raster_num
		lda #raster_6_pos
		sta $d012
		
; split 5 is done, exit irq interrupt		
					
		jmp irq_exit		
		
; raster split 6 code ------------------------------------

irq_rout6

		lda #$01			; white background for extended borders
		sta $d021

; set interrupt handler ready for split 1

		lda #$01
		sta raster_num
		lda #raster_1_pos
		sta $d012

; open the top and bottom borders please vic!
				
		lda #$14
		sta $d011

		lda #$fc
		cmp $d012
		bne *-$03

		lda #$1b
		sta $d011

; turn off all sprites - don't really have to worry since
; the sprites don't go anywhere near the borders so sprites
; near the bottom won't appear at the top, etc.  just been
; over cautious here...	
				
		lda #$00
		sta $d015
			
; set sync variable for main line code synchronisation

set_sync
		lda #$01				; set the sync flag to '1'		
		sta sync				; so the mainline code runs!
		
; restore registers and exit irq interrupt

irq_exit

		pla
		tay
		pla
		tax
		pla
nmi		rti

; end irq interrupt =============================


; subroutines ===================================


; clear the screen!

clear_screen
		lda #$01			; white background
		sta $d021
		lda #$00			; black border
		sta $d020
		tax
clsloop
		lda #$20			; clear with space character
		sta $0400,x
		sta $0400+250,x
		sta $0400+500,x
		sta $0400+750,x
		lda #$00			; black out the colour ram
		sta $d800,x
		sta $d800+250,x
		sta $d800+500,x
		sta $d800+750,x	
		inx
		cpx #$fa
		bne clsloop
		
		rts
		
; subroutine to reset the self mod for the scrollers --------

reset_scroller
		lda #<scroll_text
		sta mread+$01
		lda #>scroll_text
		sta mread+$02
		
		rts
		
reset_typer		
		lda #<typer_text
		sta typer_pos+$00
		lda #>typer_text
		sta typer_pos+$01
		
		rts
		
; subroutine that updates the twister --------------------------------

; warning, very unoptimised code lives here, but it works and I don't have
; time to optimise it... mutter, excuses, etc...

twister_updater
		lda twister_tmr				; load the twsiter timer
		inc twister_tmr				; increase by 1
		cmp #$ff					; if it $ff (about 5 seconds?)
		beq twister_change			; yes? change the twister pattern!
									; no?
		jmp twister_move			; move existing pattern then!

twister_change						; time for a pattern change
		lda twister_sel				; load which pattern we're on now
		
		cmp #$01					; ready for pattern 2?
		bne *+$05					; no? jump down and test for 3...
		jmp twister_setup2			; yes? set up pattern 2!
		
		cmp #$02					; ready for pattern 3?
		bne *+$05					; no? jump down and setup pattern 1
		jmp twister_setup3			; yes? set up pattern 3!
		
		cmp #$03					; ready for pattern 4?
		bne *+$05					; no? jump down and setup pattern 1
		jmp twister_setup4			; yes? set up pattern 4!
		
		cmp #$04					; ready for pattern 5?
		bne *+$05					; no? jump down and setup pattern 1
		jmp twister_setup5			; yes? set up pattern 5!				

; set up twister pattern 1		
		
		ldx #$00
twister1_loop1
		lda $3918,x					; copy starting chars into place for
		sta $3818,x					; pattern 1
		inx
		cpx #$98
		bne	twister1_loop1
		
		ldx #$00
twister1_loop2
		lda twister_dir1,x			; copy the direction and count data
		sta twister_dir,x			; into place for pattern 1
		lda twister_cnt1,x
		sta twister_cnt,x		
		inx
		cpx #$13
		bne	twister1_loop2			
		
		lda #$01					; set ready for twister pattern 2 when
		sta twister_sel				; when time for next change
		
		jmp reset_twister_tmr		; jump to reset the twister timer

; set up twister pattern 2		
				
twister_setup2						; same as above but for pattern 2!
		ldx #$00
twister2_loop1
		lda $3a18,x
		sta $3818,x
		inx
		cpx #$98
		bne	twister2_loop1
		
		ldx #$00
twister2_loop2
		lda twister_dir2,x
		sta twister_dir,x
		lda twister_cnt2,x
		sta twister_cnt,x		
		inx
		cpx #$13
		bne	twister2_loop2
		
		lda #$02
		sta twister_sel
		
		jmp reset_twister_tmr

; set up twister pattern 3		
				
twister_setup3						; same as above but for pattern 3!
		ldx #$00
twister3_loop
		lda $3a18,x
		sta $3818,x
		inx
		cpx #$98
		bne	twister3_loop
		
		ldx #$00
twister3_loop2
		lda twister_dir3,x
		sta twister_dir,x
		lda twister_cnt3,x
		sta twister_cnt,x		
		inx
		cpx #$13
		bne	twister3_loop2		
		
		lda #$03
		sta twister_sel
		
		jmp reset_twister_tmr		
		
; set up twister pattern 4		
				
twister_setup4						; same as above but for pattern 4!
		ldx #$00
twister4_loop
		lda $3a18,x
		sta $3818,x
		inx
		cpx #$98
		bne	twister4_loop
		
		ldx #$00
twister4_loop2
		lda twister_dir4,x
		sta twister_dir,x
		lda twister_cnt4,x
		sta twister_cnt,x		
		inx
		cpx #$13
		bne	twister4_loop2		
		
		lda #$04
		sta twister_sel

		jmp reset_twister_tmr		
				
; set up twister pattern 5		
				
twister_setup5						; same as above but for pattern 5!
		ldx #$00
twister5_loop
		lda $3a18,x
		sta $3818,x
		inx
		cpx #$98
		bne	twister5_loop
		
		ldx #$00
twister5_loop2
		lda twister_dir5,x
		sta twister_dir,x
		lda twister_cnt5,x
		sta twister_cnt,x		
		inx
		cpx #$13
		bne	twister5_loop2		
		
		lda #$00
		sta twister_sel									
		
reset_twister_tmr					; reset twister timer to count
		lda #$00					; another 5 seconds please!
		sta twister_tmr		
			
; time to animate the actual twister!  all we're really doing is
; moving each of the 19 chars up or down in order a set number
; of times to produce a wave, step or mixed pattern		
			
twister_move
				
; twister char 1

		lda twister_dir+$00			; shoudl char 1 be mving up or down?
		cmp #$01					; up?
		bne twister1down			; no? then branch to down
									; yes?
		lda $3818					; load first byte of first char into
		sta twister_tmp				; temporary store
		
		ldx #$00
twisterchar_1u
		lda $3819,x					; move each next byte upwards
		sta $3818,x					; i.e. byte 2 goes to 1, byte 3
		inx							; goes to 2, etc
		cpx #$08					; do this 8 times to make it
		bne twisterchar_1u			; look like the char 'slides' up
		
		lda twister_tmp				; finally, copy byte 1 we saved at start
		sta $381f					; into byte 8 to complete the 'slide'
		
		dec twister_cnt+$00			; decrease this chars counter
		lda twister_cnt+$00		
		cmp #$00					; is the counter 0? has it slid all the way up?
		bne *+$07					; no? brach down $07 to skip direction change code
									; counter is 0?
		lda #$00					; then load the direction store with a new
		sta twister_dir+$00			; direction of '0' which is down in this code

		jmp twister2				; now do the same for char 2!

twister1down
		lda $381f					; to move the char 'down', it's the reverse
		sta twister_tmp				; of the above code to make the char appear to
									; 'slide' down!  i.e. the bytes are copied
		ldx #$07					; in a slightly different order to simulate
twisterchar_1d						; a downwards slide...
		lda $3818-1,x
		sta $3818,x
		dex
		bpl twisterchar_1d
		
		lda twister_tmp
		sta $3818		

		inc	twister_cnt+$00
		lda twister_cnt+$00		
		cmp #$0f
		bne twister2
		
		lda #$01
		sta twister_dir+$00

; twister char 2
							
twister2							; twister char 2 is handled the same way as
									; char 1 above, just with different memory
		lda twister_dir+$01			; addresses to move the char 2 bytes instead!
		cmp #$01
		bne twister2down
		
		lda $3820
		sta twister_tmp
		
		ldx #$00
twisterchar_2u
		lda $3821,x
		sta $3820,x
		inx
		cpx #$08
		bne twisterchar_2u
		
		lda twister_tmp
		sta $3827		
		
		dec twister_cnt+$01
		lda twister_cnt+$01		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$01
	
		jmp twister3

twister2down
		lda $3827
		sta twister_tmp

		ldx #$07	
twisterchar_2d
		lda $3820-1,x
		sta $3820,x
		dex
		bpl twisterchar_2d
		
		lda twister_tmp
		sta $3820		

		inc	twister_cnt+$01
		lda twister_cnt+$01		
		cmp #$0f
		bne twister3
		
		lda #$01
		sta twister_dir+$01

; twister char 3

twister3							; and the same for char 3 and so on!!!!
		
		lda twister_dir+$02
		cmp #$01
		bne twister3down
		
		lda $3828
		sta twister_tmp
		
		ldx #$00
twisterchar_3u
		lda $3829,x
		sta $3828,x
		inx
		cpx #$08
		bne twisterchar_3u
		
		lda twister_tmp
		sta $382f		
		
		dec twister_cnt+$02
		lda twister_cnt+$02		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$02
	
		jmp twister4

twister3down
		lda $382f
		sta twister_tmp

		ldx #$07	
twisterchar_3d
		lda $3828-1,x
		sta $3828,x
		dex
		bpl twisterchar_3d
		
		lda twister_tmp
		sta $3828		

		inc	twister_cnt+$02
		lda twister_cnt+$02		
		cmp #$0f
		bne twister4
		
		lda #$01
		sta twister_dir+$02

; twister char 4

twister4								; yes, and for char 4...
										; i am not repeating myself
		lda twister_dir+$03				; all the way through
		cmp #$01						; to char 19!
		bne twister4down
		
		lda $3830
		sta twister_tmp
		
		ldx #$00
twisterchar_4u
		lda $3831,x
		sta $3830,x
		inx
		cpx #$08
		bne twisterchar_4u
		
		lda twister_tmp
		sta $3837		
		
		dec twister_cnt+$03
		lda twister_cnt+$03		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$03
	
		jmp twister5

twister4down
		lda $3837
		sta twister_tmp

		ldx #$07	
twisterchar_4d
		lda $3830-1,x
		sta $3830,x
		dex
		bpl twisterchar_4d
		
		lda twister_tmp
		sta $3830		

		inc	twister_cnt+$03
		lda twister_cnt+$03		
		cmp #$0f
		bne twister5
		
		lda #$01
		sta twister_dir+$03

; twister char 5

twister5								; i said no repeating myself...

		lda twister_dir+$04
		cmp #$01
		bne twister5down
		
		lda $3838
		sta twister_tmp
		
		ldx #$00
twisterchar_5u
		lda $3839,x
		sta $3838,x
		inx
		cpx #$08
		bne twisterchar_5u
		
		lda twister_tmp
		sta $383f		
		
		dec twister_cnt+$04
		lda twister_cnt+$04		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$04
	
		jmp twister6

twister5down
		lda $383f
		sta twister_tmp

		ldx #$07	
twisterchar_5d
		lda $3838-1,x
		sta $3838,x
		dex
		bpl twisterchar_5d
		
		lda twister_tmp
		sta $3838		

		inc	twister_cnt+$04
		lda twister_cnt+$04		
		cmp #$0f
		bne twister6
		
		lda #$01
		sta twister_dir+$04

; twister char 6

twister6								; you're still checking?
		
		lda twister_dir+$05
		cmp #$01
		bne twister6down
		
		lda $3840
		sta twister_tmp
		
		ldx #$00
twisterchar_6u
		lda $3841,x
		sta $3840,x
		inx
		cpx #$08
		bne twisterchar_6u
		
		lda twister_tmp
		sta $3847		
		
		dec twister_cnt+$05
		lda twister_cnt+$05		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$05
	
		jmp twister7

twister6down
		lda $3847
		sta twister_tmp

		ldx #$07	
twisterchar_6d
		lda $3840-1,x
		sta $3840,x
		dex
		bpl twisterchar_6d
		
		lda twister_tmp
		sta $3840		

		inc	twister_cnt+$05
		lda twister_cnt+$05		
		cmp #$0f
		bne twister7
		
		lda #$01
		sta twister_dir+$05

; twister char 7

twister7								; nope, no repeats here...

		lda twister_dir+$06
		cmp #$01
		bne twister7down
		
		lda $3848
		sta twister_tmp
		
		ldx #$00
twisterchar_7u
		lda $3849,x
		sta $3848,x
		inx
		cpx #$08
		bne twisterchar_7u
		
		lda twister_tmp
		sta $384f		
		
		dec twister_cnt+$06
		lda twister_cnt+$06		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$06
	
		jmp twister8

twister7down
		lda $384f
		sta twister_tmp

		ldx #$07	
twisterchar_7d
		lda $3848-1,x
		sta $3848,x
		dex
		bpl twisterchar_7d
		
		lda twister_tmp
		sta $3848		

		inc	twister_cnt+$06
		lda twister_cnt+$06		
		cmp #$0f
		bne twister8
		
		lda #$01
		sta twister_dir+$06
		
; twister char 8

twister8								; seriously now?
		
		lda twister_dir+$07
		cmp #$01
		bne twister8down
		
		lda $3850
		sta twister_tmp
		
		ldx #$00
twisterchar_8u
		lda $3851,x
		sta $3850,x
		inx
		cpx #$08
		bne twisterchar_8u
		
		lda twister_tmp
		sta $3857		
		
		dec twister_cnt+$07
		lda twister_cnt+$07		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$07
	
		jmp twister9

twister8down
		lda $3857
		sta twister_tmp

		ldx #$07	
twisterchar_8d
		lda $3850-1,x
		sta $3850,x
		dex
		bpl twisterchar_8d
		
		lda twister_tmp
		sta $3850		

		inc	twister_cnt+$07
		lda twister_cnt+$07		
		cmp #$0f
		bne twister9
		
		lda #$01
		sta twister_dir+$07		

; twister char 9

twister9								; yawn....

		lda twister_dir+$08
		cmp #$01
		bne twister9down
		
		lda $3858
		sta twister_tmp
		
		ldx #$00
twisterchar_9u
		lda $3859,x
		sta $3858,x
		inx
		cpx #$08
		bne twisterchar_9u
		
		lda twister_tmp
		sta $385f		
		
		dec twister_cnt+$08
		lda twister_cnt+$08		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$08
	
		jmp twister10

twister9down
		lda $385f
		sta twister_tmp

		ldx #$07	
twisterchar_9d
		lda $3858-1,x
		sta $3858,x
		dex
		bpl twisterchar_9d
		
		lda twister_tmp
		sta $3858		

		inc	twister_cnt+$08
		lda twister_cnt+$08		
		cmp #$0f
		bne twister10
		
		lda #$01
		sta twister_dir+$08	

; twister char 10

twister10								; ZZZZZZzzzzzzzz.......

		lda twister_dir+$09
		cmp #$01
		bne twister10down
		
		lda $3860
		sta twister_tmp
		
		ldx #$00
twisterchar_10u
		lda $3861,x
		sta $3860,x
		inx
		cpx #$08
		bne twisterchar_10u
		
		lda twister_tmp
		sta $3867
		
		dec twister_cnt+$09
		lda twister_cnt+$09		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$09
	
		jmp twister11

twister10down
		lda $3867
		sta twister_tmp

		ldx #$07	
twisterchar_10d
		lda $3860-1,x
		sta $3860,x
		dex
		bpl twisterchar_10d
		
		lda twister_tmp
		sta $3860		

		inc	twister_cnt+$09
		lda twister_cnt+$09		
		cmp #$0f
		bne twister11
		
		lda #$01
		sta twister_dir+$09	

; twister char 11

twister11								; 9 chars left to 'animate'...

		lda twister_dir+$0a
		cmp #$01
		bne twister11down
		
		lda $3868
		sta twister_tmp
		
		ldx #$00
twisterchar_11u
		lda $3869,x
		sta $3868,x
		inx
		cpx #$08
		bne twisterchar_11u
		
		lda twister_tmp
		sta $386f
		
		dec twister_cnt+$0a
		lda twister_cnt+$0a		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$0a
	
		jmp twister12

twister11down
		lda $386f
		sta twister_tmp

		ldx #$07	
twisterchar_11d
		lda $3868-1,x
		sta $3868,x
		dex
		bpl twisterchar_11d
		
		lda twister_tmp
		sta $3868		

		inc	twister_cnt+$0a
		lda twister_cnt+$0a		
		cmp #$0f
		bne twister12
		
		lda #$01
		sta twister_dir+$0a	

; twister char 12

twister12							; no, i'm not counting them down now...

		lda twister_dir+$0b
		cmp #$01
		bne twister12down
		
		lda $3870
		sta twister_tmp
		
		ldx #$00
twisterchar_12u
		lda $3871,x
		sta $3870,x
		inx
		cpx #$08
		bne twisterchar_12u
		
		lda twister_tmp
		sta $3877
		
		dec twister_cnt+$0b
		lda twister_cnt+$0b		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$0b
	
		jmp twister13

twister12down
		lda $3877
		sta twister_tmp

		ldx #$07	
twisterchar_12d
		lda $3870-1,x
		sta $3870,x
		dex
		bpl twisterchar_12d
		
		lda twister_tmp
		sta $3870		

		inc	twister_cnt+$0b
		lda twister_cnt+$0b		
		cmp #$0f
		bne twister13
		
		lda #$01
		sta twister_dir+$0b	

; twister char 13

twister13							; really not counting down, but 7 left
									; to do if you're interested...
		lda twister_dir+$0c
		cmp #$01
		bne twister13down
		
		lda $3878
		sta twister_tmp
		
		ldx #$00
twisterchar_13u
		lda $3879,x
		sta $3878,x
		inx
		cpx #$08
		bne twisterchar_13u
		
		lda twister_tmp
		sta $387f
		
		dec twister_cnt+$0c
		lda twister_cnt+$0c		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$0c
	
		jmp twister14

twister13down
		lda $387f
		sta twister_tmp

		ldx #$07	
twisterchar_13d
		lda $3878-1,x
		sta $3878,x
		dex
		bpl twisterchar_13d
		
		lda twister_tmp
		sta $3878

		inc	twister_cnt+$0c
		lda twister_cnt+$0c		
		cmp #$0f
		bne twister14
		
		lda #$01
		sta twister_dir+$0c	

; twister char 14

twister14							; look! char manipulation with no hands!

		lda twister_dir+$0d
		cmp #$01
		bne twister14down
		
		lda $3880
		sta twister_tmp
		
		ldx #$00
twisterchar_14u
		lda $3881,x
		sta $3880,x
		inx
		cpx #$08
		bne twisterchar_14u
		
		lda twister_tmp
		sta $3887
		
		dec twister_cnt+$0d
		lda twister_cnt+$0d		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$0d
	
		jmp twister15

twister14down
		lda $3887
		sta twister_tmp

		ldx #$07	
twisterchar_14d
		lda $3880-1,x
		sta $3880,x
		dex
		bpl twisterchar_14d
		
		lda twister_tmp
		sta $3880

		inc	twister_cnt+$0d
		lda twister_cnt+$0d		
		cmp #$0f
		bne twister15
		
		lda #$01
		sta twister_dir+$0d	

; twister char 15

twister15								; wow, you're still reading?!?!

		lda twister_dir+$0e
		cmp #$01
		bne twister15down
		
		lda $3888
		sta twister_tmp
		
		ldx #$00
twisterchar_15u
		lda $3889,x
		sta $3888,x
		inx
		cpx #$08
		bne twisterchar_15u
		
		lda twister_tmp
		sta $388f
		
		dec twister_cnt+$0e
		lda twister_cnt+$0e		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$0e
	
		jmp twister16

twister15down
		lda $388f
		sta twister_tmp

		ldx #$07	
twisterchar_15d
		lda $3888-1,x
		sta $3888,x
		dex
		bpl twisterchar_15d
		
		lda twister_tmp
		sta $3888

		inc	twister_cnt+$0e
		lda twister_cnt+$0e		
		cmp #$0f
		bne twister16
		
		lda #$01
		sta twister_dir+$0e

; twister char 16

twister16								; nothing better to do?

		lda twister_dir+$0f
		cmp #$01
		bne twister16down
		
		lda $3890
		sta twister_tmp
		
		ldx #$00
twisterchar_16u
		lda $3891,x
		sta $3890,x
		inx
		cpx #$08
		bne twisterchar_16u
		
		lda twister_tmp
		sta $3897
		
		dec twister_cnt+$0f
		lda twister_cnt+$0f		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$0f
	
		jmp twister17

twister16down
		lda $3897
		sta twister_tmp

		ldx #$07	
twisterchar_16d
		lda $3890-1,x
		sta $3890,x
		dex
		bpl twisterchar_16d
		
		lda twister_tmp
		sta $3890

		inc	twister_cnt+$0f
		lda twister_cnt+$0f		
		cmp #$0f
		bne twister17
		
		lda #$01
		sta twister_dir+$0f

; twister char 17

twister17							; should've done more with your life
									; than reading code comments?
		lda twister_dir+$10
		cmp #$01
		bne twister17down
		
		lda $3898
		sta twister_tmp
		
		ldx #$00
twisterchar_17u
		lda $3899,x
		sta $3898,x
		inx
		cpx #$08
		bne twisterchar_17u
		
		lda twister_tmp
		sta $389f
		
		dec twister_cnt+$10
		lda twister_cnt+$10		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$10
	
		jmp twister18

twister17down
		lda $389f
		sta twister_tmp

		ldx #$07	
twisterchar_17d
		lda $3898-1,x
		sta $3898,x
		dex
		bpl twisterchar_17d
		
		lda twister_tmp
		sta $3898

		inc	twister_cnt+$10
		lda twister_cnt+$10		
		cmp #$0f
		bne twister18
		
		lda #$01
		sta twister_dir+$10	
		
; twister char 18

twister18							; this is getting sad now...
		
		lda twister_dir+$11
		cmp #$01
		bne twister18down
		
		lda $38a0
		sta twister_tmp
		
		ldx #$00
twisterchar_18u
		lda $38a1,x
		sta $38a0,x
		inx
		cpx #$08
		bne twisterchar_18u
		
		lda twister_tmp
		sta $38a7
		
		dec twister_cnt+$11
		lda twister_cnt+$11		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$11
	
		jmp twister19

twister18down
		lda $38a7
		sta twister_tmp

		ldx #$07	
twisterchar_18d
		lda $38a0-1,x
		sta $38a0,x
		dex
		bpl twisterchar_18d
		
		lda twister_tmp
		sta $38a0

		inc	twister_cnt+$11
		lda twister_cnt+$11		
		cmp #$0f
		bne twister19
		
		lda #$01
		sta twister_dir+$11		
	
; twister char 19

twister19							; last one now, thank goodness... ;)

		lda twister_dir+$12
		cmp #$01
		bne twister19down
		
		lda $38a8
		sta twister_tmp
		
		ldx #$00
twisterchar_19u
		lda $38a9,x
		sta $38a8,x
		inx
		cpx #$08
		bne twisterchar_19u
		
		lda twister_tmp
		sta $38af
		
		dec twister_cnt+$12
		lda twister_cnt+$12		
		cmp #$00
		bne *+$07
		
		lda #$00
		sta twister_dir+$12
	
		jmp twister_dun

twister19down
		lda $38af
		sta twister_tmp

		ldx #$07	
twisterchar_19d
		lda $38a8-1,x
		sta $38a8,x
		dex
		bpl twisterchar_19d
		
		lda twister_tmp
		sta $38a8

		inc	twister_cnt+$12
		lda twister_cnt+$12		
		cmp #$0f
		bne twister_dun
		
		lda #$01
		sta twister_dir+$12	
		
twister_dun								; we're done with char manipulation now!
										; damn! there *was* 1 more comment!
		rts
				
					
; subroutine that type writes the greet text on the screen --------- 
	
type_writer
		ldx typer_x
		cpx #$26
		bcs typer_cnt

; fetch a character from the text table

typer_read
		ldy #$00
		lda (typer_pos),y
		bne typer_write
		
		jsr reset_typer
		
		jmp typer_read

; write that character to the screen
		
typer_write
		sta $07c1,x

; update the text position

		inc typer_pos+$00
		bne *+$04
		inc typer_pos+$01

; store the current text position		
		
typer_cnt
		inx
		stx typer_x
		
		rts

; irq sync wait subroutine ---------------------------------- 
	   	 
sync_wait								; wait for sync update in irq code!
		lda #$00						; waits until the sync flag is
		sta sync						; set at the end of the interupt
sw_loop									; code above
		cmp sync						; no code in the main loop above
		beq sw_loop						; is executed until the interupt
										; has run it's course!
		rts
		   	
; end subroutines ===============================


; lovely jubbly data lives here! ==========================

; sprites behind the logo - positions, definitions, etc.

logo_sprites_x		!byte $1f,$5f,$8f,$bf,$ff
logo_sprites_y		!byte $3b
logo_sprites_def	!byte $a8,$a9,$aa,$ab,$ac
logo_sprites_s		!byte $05,$04,$04,$04,$04
logo_sprites_msb	!byte $00
logo_sprites_ydir	!byte $00

twister_sprites_x		!byte $40,$58,$70,$40,$70,$40,$58,$70
twister_sprites_y		!byte $83,$83,$83,$97,$97,$ac,$ac,$ac
twister_sprites_def		!byte $a0,$a1,$a2,$a3,$a4,$a5,$a6,$a7
twister_sprites_col		!byte $00,$00,$00,$00,$00,$00,$00,$00
twister_sprites_sl		!byte $03,$02,$02,$02,$02,$02,$02,$02
twister_sprites_sr		!byte $01,$02,$02,$02,$02,$02,$02,$02
twister_sprites_msb		!byte $00
twister_sprites_xdir	!byte $00

fader_sprites_x		!byte $39,$1d
fader_sprites_y		!byte $d3,$d3
fader_sprites_def	!byte $ae,$ad
fader_sprites_col	!byte $01,$01

; matrix data for the cosine logo setup

logo_matrix
!byte $20,$6C,$6D,$4F,$8C,$2F,$6F,$03,$43,$6F,$44,$62,$20,$72,$4F,$4E,$4E,$42,$0E,$0F,$43,$4D,$4C,$6C,$6D,$15,$16,$4D,$4E,$4F,$6C,$1B,$1C,$1D,$20,$55,$6D,$B2,$92,$20
!byte $20,$5C,$20,$97,$98,$21,$22,$23,$24,$25,$26,$27,$28,$29,$2A,$5B,$2C,$2D,$23,$23,$30,$97,$32,$33,$34,$35,$36,$37,$38,$39,$3A,$3B,$23,$3D,$3E,$98,$5C,$97,$98,$20
!byte $20,$20,$20,$20,$40,$41,$23,$23,$23,$45,$46,$23,$23,$23,$4A,$4B,$23,$23,$23,$23,$50,$51,$52,$53,$54,$23,$23,$57,$58,$59,$5A,$23,$23,$23,$5E,$20,$20,$20,$20,$20
!byte $20,$47,$48,$49,$60,$61,$23,$23,$23,$65,$66,$23,$68,$23,$23,$6B,$23,$23,$23,$23,$70,$71,$23,$73,$74,$75,$23,$77,$78,$79,$7A,$23,$7C,$7D,$7E,$20,$47,$48,$A7,$20
!byte $20,$67,$69,$A9,$20,$81,$23,$83,$23,$85,$86,$87,$88,$89,$8A,$8B,$23,$8D,$8E,$8F,$90,$91,$23,$93,$94,$95,$23,$23,$23,$99,$9A,$9B,$9C,$9D,$9E,$20,$67,$69,$A9,$20
!byte $20,$20,$20,$20,$20,$A1,$A2,$A3,$A4,$A5,$A6,$23,$A8,$23,$AA,$AB,$AC,$AD,$AE,$AF,$B0,$B1,$23,$B3,$B4,$B5,$B6,$23,$B8,$B9,$BA,$BB,$BC,$BD,$BE,$20,$20,$20,$20,$20
!byte $20,$20,$20,$5B,$20,$C1,$C2,$C3,$C4,$C5,$C6,$C7,$C8,$C9,$CA,$CB,$CC,$CD,$CE,$CF,$D0,$D1,$D2,$D3,$D4,$D5,$D6,$D7,$D8,$20,$97,$DB,$DC,$DD,$DE,$DF,$20,$20,$20,$20
!byte $20,$84,$4F,$6E,$56,$2F,$76,$20,$96,$5C,$20,$44,$63,$6F,$6D,$44,$6C,$4C,$20,$20,$62,$4C,$42,$2E,$6C,$6D,$20,$2E,$2F,$4C,$20,$20,$4F,$6F,$64,$6C,$2E,$82,$43,$20
!byte $E0,$3C,$E0,$E0,$E0,$5D,$E0,$E0,$E0,$E0,$E0,$E0,$E0,$E0,$B7,$3C,$E0,$E0,$E0,$E0,$5D,$E0,$E0,$E0,$3C,$E0,$E0,$B7,$5D,$E0,$E0,$E0,$E0,$E0,$B7,$E0,$E0,$3C,$E0,$E0

; matrix data for the initial twister setup

twister_matrix
!byte $03,$03,$03,$04,$04,$05,$05,$06,$06,$07,$07,$08,$08,$09,$09,$0A,$0A,$0B,$0B,$0C,$0C,$0D,$0D,$0E,$0E,$0F,$0F,$10,$10,$11,$11,$12,$12,$13,$13,$14,$14,$15,$15

; data store for the twister

twister_dir !byte $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01	; store that is displayed
twister_cnt !byte $17,$18,$19,$1a,$1b,$1c,$1d,$1e,$1f,$20,$21,$22,$23,$24,$25,$26,$27,$28,$29

twister_dir1 !byte $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01	; pattern 1 - wave
twister_cnt1 !byte $17,$18,$19,$1a,$1b,$1c,$1d,$1e,$1f,$20,$21,$22,$23,$24,$25,$26,$27,$28,$29

twister_dir2 !byte $00,$01,$00,$01,$00,$01,$00,$01,$00,$01,$00,$01,$00,$01,$00,$01,$00,$01,$00	; pattern 2	- step
twister_cnt2 !byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00

twister_dir3 !byte $00,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01	; pattern 3 - mix of both
twister_cnt3 !byte $00,$02,$04,$06,$08,$0a,$0c,$0e,$10,$12,$14,$16,$18,$1a,$1c,$1e,$20,$22,$24

twister_dir4 !byte $00,$00,$00,$00,$01,$01,$01,$01,$00,$00,$00,$01,$01,$01,$01,$00,$00,$00,$00	; pattern 4 - left and right alternate
twister_cnt4 !byte $07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07

twister_dir5 !byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; pattern 5 - all bars equal
twister_cnt5 !byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00

twister_tmp	!byte $00

; double size character font offsets and widths

char_pos_dcd
		!byte $40,$42,$44,$46,$48,$4a,$4c,$4e		; @ to G
		!byte $50,$52,$54,$56,$58,$5a,$5c,$5e		; H to O
		!byte $80,$82,$84,$86,$88,$8a,$8c,$8e		; P to W
		!byte $90,$92,$94,$00,$00,$00,$00,$00		; X to Z, 5 * punct.
		!byte $40,$96,$00,$00,$00,$00,$00,$97		; space to '
		!byte $98,$9a,$00,$00,$9d,$9e,$9c,$c0		; ( to /
		!byte $c2,$c4,$c6,$c8,$ca,$cc,$ce,$d0		; 0 to 7
		!byte $d2,$d4,$d6,$d7,$00,$00,$00,$d8		; 8 to ?

char_width_dcd
		!byte $00,$01,$01,$01,$01,$01,$01,$01		; @ to G
		!byte $01,$00,$01,$01,$01,$01,$01,$01		; H to O
		!byte $01,$01,$01,$01,$01,$01,$01,$01		; P to W
		!byte $01,$01,$01,$01,$01,$01,$01,$01		; X to Z, 5 * punct.
		!byte $00,$00,$01,$01,$01,$01,$01,$00		; space to '
		!byte $01,$01,$01,$01,$00,$01,$00,$01		; ( to /
		!byte $01,$01,$01,$01,$01,$01,$01,$01		; 0 to 7
		!byte $01,$01,$00,$00,$01,$01,$01,$01		; 8 to ?
		
; colours for the greets text typer

greets_colour
		!byte $01,$0f,$0c,$0b,$00,$0b,$0c,$0f				
		
; text for the scrolling message

scroll_text
		!scr $43,"              cosine systems presents..."
		!scr "              "
				
		!scr $41,"md202006 - get well soon "
		!scr "         "

		!scr $42,"coding and graphics by... ",$41,"andy "
		!scr "         "		
		!scr $42,"music by... ",$41,"odie "
		!scr "         "
		!scr $42,"testing by... ",$41,"enigma "
		!scr "         "		

		!scr $42,"message from andy:  "
		!scr $43,"this simple little demo is dedicated to t.m.r "		
		!scr "who has been really rather ill in hospital with "
		!scr "this damn covid-19 virus and i hope it will cheer "
		!scr "him up a little!   i've borrowed (stole) the "
		!scr "naming convention he used in his monthly demo series " 
		!scr "as a nod to those excellent little releases and "
		!scr "i've done my best to match the 'style' of them, "		
		!scr "although at this early stage in my coding career, "		
		!scr "i obviously can't match t.m.r's code wrangling..."
		!scr "        "
		
		!scr "thanks to odie for providing a tune so quickly "
		!scr "after being asked!   the music was originally "
		!scr "written with an 8580 sid in mind, but the code should "
		!scr "detect a 6581 sid and modify some of the filter tables "												
		!scr "to at least make it audible.   if the tune sounds "
		!scr "familiar it may be because it's a cover of an "
		!scr "amiga mod called ",$41,"'firepower' by jellybean... "  
		!scr "        "		
		
		!scr $43,"before i hand the keyboard to a few other people, "
		!scr "i just want to wish t.m.r well and ask that he "
		!scr "regains his strength as soon as possible because, "
		!scr "whether he likes it or not, he's kind of become my "
		!scr "coding mentor and there's so much more i want "
		!scr "to learn!   get well soon!!!"						 									
		!scr "        "
		
		!scr $42,"now, some words from other members of cosine... "
		!scr "            "	
		
		!scr "message from odie:  "
		!scr $43,"yes, poor t.m.r has been very, very ill with "
		!scr "this stupid covid-19.   i get more angered with "
		!scr "people who don't believe they will get it and that "
		!scr "don't observe the rules.   i have been in touch with "	
		!scr "t.m.r's wife several times per week since he's been "					
		!scr "in hospital.   now that he is starting to get better "			
		!scr "more rapidly, we should have him back coding sometime "			
		!scr "soon.   yes, he has been a bit of a grumpy fart with "	
		!scr "the nurses on all accounts from being prodded and "	
		!scr "poked, though i thought he liked that sort of "
		!scr "treatment.   ;)   some day when the arcades re-open, "	
		!scr "we are all going to have a trip to blackpool to "	
		!scr "have some good old fashioned 80's arcade fun.   "							
		!scr "will look forward to that day, but the main thing "			
		!scr "is you get well soon...   "			
		!scr "        "								
		
		!scr $42,"message from enigma:  "
		!scr $43,"hi jason, seems like the only thing i am good "
		!scr "for at the moment is scroll texts!   i need to "
		!scr "pull my finger out soon and do some graphics for "
		!scr "cosine won't i?   or maybe even actually learn "
		!scr "some assembly! ;)   hope you are looking after "
		!scr "yourself and what a lovely idea by andy to do a "
		!scr "demo?   looking forward to catching up again with "
		!scr "you soon when things calm down.   take care! "
		!scr "        "
		
		!scr $42,"message from soren:  "
		!scr $43,"dear t.m.r, i wish you all the best and soon recovery!   "
		!scr "i recently got very ill due to medication that my body "			
		!scr "didn't like, but i just stayed home to recover and i am "
		!scr "ok again.   but i hope you're in good hands until we meet "
		!scr "online again.   take care and get well soon mate. :)"
		!scr "        "
		
		!scr $42,"message from iapetus:  "
		!scr $43,"jason my friend, wishing you a speedy recovery "
		!scr "from portugal... "
		!scr "        "														
				
		!scr $42,"next, some words from people on csdb... "
		!scr "            "

		!scr $43,"message from moloch/triad:  "
		!scr "hiya jason, positive thoughts for you as you work toward "
		!scr "a full recovery!   best wishes friend... "
		!scr "        "			
				
		!scr "message from motion/artstate:  "
		!scr "hi t.m.r, sorry to hear you've been feeling incredibly "
		!scr "crook (excuse the kiwi slang!).   once you're up "
		!scr "to speed (...and not on it), there are demos to be made "
		!scr "and they don't make themselves!   all the very best! "
		!scr "        "
		
		!scr "message from fuzz/retro64:  "
		!scr "sending you positive vibes, 8 bits at a time.   fingers "
		!scr "crossed for a full recovery! "
		!scr "        "		
		
		!scr "message from trooper:  "
		!scr "hey jason, glad to hear you are on the mend and on the "
		!scr "road to recovery.   the quicker you get back to full "
		!scr "strength, the quicker you can get all your websites "
		!scr "back up to give people something to read.   take care. "
		!scr "        "
		
		!scr "message from cadaver/covertbitops:  "
		!scr "get well soon!   isn't it time for cosine's crap game "
		!scr "compo to return, for proper crap?   (craptastic compo "
		!scr "entries are too good and limited to 4kb) "
		!scr "        "					
	
		!scr "message from jazzcat/onslaught:  "
		!scr "hi t.m.r, you old bugger... how're things now?   "
		!scr "hopefully better and you are on the mend.   what an "
		!scr "ordeal eh?   now you're out of the woods, time for "
		!scr "payback on this damn covid-19?   maybe a little demo "
		!scr "or game?   would be fun :)   when i heard you had this, "
		!scr "my heart sank, but then i read you had recovered "
		!scr "but have spent much time in hospital.   well, "			
		!scr "sending my best wishes.   chat soon mate..."
		!scr "        "
		
		!scr "message from ian coog/hvsc:  "
		!scr "i wish you a full recovery.   stay strong and come "
		!scr "back soon,  the scene needs you back 100 per cent.   "							
		!scr "hugs to you and your family. "
		!scr "        "		
						
		!scr "message from richard/tnd:  "
		!scr "after hearing the news announced on csdb, it was sad "
		!scr "to hear that you fell victim to covid-19.   you have "
		!scr "entertained the c64 scene with stunning games and "
		!scr "demos.   i also remember joining you and kenz down at "
		!scr "the very first revival checking out some of your game "
		!scr "productions.   i hope you get well soon and stay safe! "	
		!scr "        "
		
		!scr "message from redback/arkanix labs:  "
		!scr "t.m.r, i always enjoy your quirky comments and sincere "
		!scr "manner.   get well mate! "
		!scr "        "			

		!scr $42,"now some words from people on the plus/4 world forum... "
		!scr "            "
		
		!scr $43,"message from luca/fire:  "
		!scr "ok, ok, we see that you tried to wriggle out, don't try "
		!scr "to fool us with this extreme attempt to avoid releasing "
		!scr "your new productions for the 264 series!   get well ubersoon, "
		!scr "we're now waiting for your 'returntro' on plus/4!   ;) "					
		!scr "        "		
		
		!scr "message from mad/bauknecht:  "
		!scr "sorry to hear!   i always adored your productions, cosine "
		!scr "is a quality label anyways!   hope you get well soonish.   " 
		!scr "good to hear you're on the mend. "
		!scr "        "
		
		!scr "message from csabo/lod:  "
		!scr "picklefishlips!   didn't expect that, did you?   wanted to "
		!scr "say something other than sorry-friend-hope-you-get-better-"
		!scr "love-your-releases.   well, now i said it...   :) "
		!scr "        "					

		!scr "message from chronos/absence:  "
		!scr "fall seven times, stand up eight - bits!   get well soon!!! "
		!scr "        "
		
		!scr "message from gerliczer:  "
		!scr "i started to get worried sick.   unfortunately, i was almost "
		!scr "right.   but now, everything is cheery again.   get well soon.   "						
		!scr "show us again how things are done.   i love your blogs and "	
		!scr "the little thingies you make. "
		!scr "            "
						
		!scr $42,"finally, some words from people on atariage forum... "
		!scr "            "
		
		!scr $43,"message from paul:  "
		!scr "don't steal from here again! "
		!scr "        "
		
		!scr "message from vince:  "
		!scr "keep fighting jason!   and don't forget your "
		!scr "special moves! "
		!scr "        "
		
		!scr "message from miker:  "
		!scr "get well t.m.r! "
		!scr "        "
		
		!scr "message from philsan:  "
		!scr "miss seeing you on atariage and retro gamer.   "
		!scr "get well soon! "	
		!scr "        "
		
		!scr "message from snicklin:  "
		!scr "everyone from atariage is wishing you well! "
		!scr "            "											
		
		!scr $42,"end of messages... "
		!scr "                       "
		!scr "scroller wraps now...  "
		!scr "                       "
			
		!byte $00	; end of text marker
		
typer_text
		!scr "                                      "
		!scr "  during this crazy year called 2020  "
		!scr " cosine very much wishes to send warm "		
		!scr "  but socially responsible hugs with  "
		!scr "  truly heartfelt best wishes to the  "		
		!scr " following lovely groups or people... "
		!scr "                                      "
		!scr "   *  a r k a n i x    l a b s  *     "			
		!scr "  *  absence  *  abyss connection  *  "
		!scr "        *  artstate digital  *        "  
		!scr "      *  ate bit  *  atlantis  *      "
		!scr "    *  booze design  *  camelot  *    "				
		!scr "    *  censor design  *  chorus  *    "
		!scr "    *  chrome  *  cncd  *  cpu  *     "
		!scr "       *  crescent  *  crest  *       "										
		!scr "  * covert bitops * defence force *   "			
		!scr "  *  dekadence  *  desire  *  dac  *  "
		!scr "      *  dmagic  *  dual crew  *      "					
		!scr "   *  exclusive on  *  fairlight  *   "
		!scr "    *  f4cg  * fire  *  flat 3  *     "								
		!scr "     *  focus  *  french touch  *     "
		!scr "  * genesis project * gheymaid inc *  "
		!scr "       *  hitmen  *  hoaxers  *       "		
		!scr "  * hokuto force * legion of doom *   "
		!scr " *  level 64  *  maniacs of noise  *  "				
		!scr "      *  mayday  *  meanteam  *       "			
		!scr "     *  metalvotze  *  noname  *      "			
		!scr "      *  nostalgia  *  nuance  *      "			
		!scr "     *  offence  *  onslaught  *      "			
		!scr "    *  orb  *  oxyron  *  padua  *    "		
		!scr "      *  performers  *  plush  *      "			
		!scr "       *  ppcs  *  psytronik  *       "				
		!scr "      *  reptilia  *  resource  *     "	
		!scr "    *  rgcd  *  secure  *  shape  *   "			
		!scr "      *  side b  *  singular  *       "			
		!scr "      *  slash  *  slipstream  *      "			
		!scr "   *  success and trc  *  style  *    "			
		!scr " *  suicyco industries  *  taquart  * "			
		!scr "   * tempest * tek * tnd * triad *    "					
		!scr "  * tristar and red sector * viruz *  "			
		!scr "   * vision * wow * wrath designs *   "	
		!scr "        *  xenon  *  xentax  *        "
		!scr "and anyone else we may have forgotton!"
		!scr " *  we  love  you  one  and  all!!  * "
		!scr "                                      "		
		
		!byte $00		; end of text marker				